/*
	minimal IF-SEGA driver
	Copyright 1999-2000 (C) ITO, Takayuki. All rights reserved.
*/

#include <drivers/KernelExport.h>
#include <drivers/module.h>
#include <kernel/OS.h>
#include <support/Debug.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include "joystick_driver.h"
#include "saturn.h"

/** "configure" might change the "info" if it auto-detects a device */

static int saturn_configure
	(int port,
	joystick_module_info *info,
	size_t size,
	saturn_cookie **out_cookie)
{
	ASSERT (info != NULL);
	ASSERT (out_cookie != NULL);

	dprintf (MODULE_NAME ": configure()\n");
	dprintf (MODULE_NAME ": module_name = %s\n", info->module_name);
	dprintf (MODULE_NAME ": device_name = %s\n", info->device_name);
	dprintf (MODULE_NAME ": %d axes, %d buttons, %d hats\n",
		info->num_axes, info->num_buttons, info->num_hats);
	*out_cookie = malloc (sizeof (saturn_cookie));
	return B_OK;
}

/** "read" actual data from device into "data" */

static int saturn_read
	(saturn_cookie *cookie,
	int port,
	extended_joystick *data,
	size_t size)
{
	static const int ampl [4] = { 0, 32767, -32768, 0 };

	ASSERT (cookie != NULL);
	ASSERT (data != NULL);
	ASSERT (size >= sizeof (extended_joystick));

	data->timestamp = system_time ();
	data->axes [0] = ampl [cookie->hat >> 2];
	data->axes [1] = ampl [cookie->hat & 3];

	/* L---RXYZSACB => ---SZYXCBALR */
	data->buttons = ~(
		((cookie->buttons & 0x800) >> 10) |	// L
		((cookie->buttons & 0x080) >>  7) |	// R
		((cookie->buttons & 0x040) >>  1) |	// X
		((cookie->buttons & 0x020) <<  1) |	// Y
		((cookie->buttons & 0x010) <<  3) |	// Z
		((cookie->buttons & 0x008) <<  5) |	// Start
		((cookie->buttons & 0x004)      ) |	// A
		((cookie->buttons & 0x003) <<  3));	// CB
	return size;
}

/** "crumble" the cookie (deallocate) when done */

static int saturn_crumble
	(saturn_cookie *cookie,
	int port)
{
	ASSERT (cookie != NULL);
	dprintf (MODULE_NAME ": crumble()\n");
	free (cookie);
	return B_OK;
}

/** "force" tells the joystick to exert force
	on the same axes as input for the specified duration */

static int saturn_force
	(saturn_cookie *cookie,
	int port,
	bigtime_t duration,
	extended_joystick *force,
	size_t size)
{
	ASSERT (cookie != NULL);
	ASSERT (force != NULL);
	ASSERT (size >= sizeof (extended_joystick));
	dprintf (MODULE_NAME ": force()\n");
	return B_ERROR;
}

static status_t std_ops
	(int32 op, ...)
{
	switch (op)
	{
	case B_MODULE_INIT:
		dprintf (MODULE_NAME ": B_MODULE_INIT\n");
		return B_OK;
	case B_MODULE_UNINIT:
		dprintf (MODULE_NAME ": B_MODULE_UNINIT\n");
		return B_OK;
	default:
		dprintf (MODULE_NAME ": std_ops (%d)\n", op);
		return B_ERROR;
	}
}

static joystick_module saturn_info =
{
	/* module_info minfo = */
	{ MODULE_PATH, 0, std_ops },
	saturn_configure,
	saturn_read,
	saturn_crumble,
	saturn_force,
	0
};

_EXPORT module_info *modules [] =
{
	(module_info *) &saturn_info, NULL
};
